@@ -29,20 +29,23 @@ from collections import OrderedDict
 from xml.etree import ElementTree
 import lxml.etree
 
-from elementpath import ElementPathError, XPath2Parser, XPathContext, XPathNode
 import elementpath
 import xmlschema
 
+from elementpath import ElementPathError, XPath2Parser, XPathContext, XPathNode
+from elementpath.xpath_token import XPathFunction
+from elementpath.datatypes import AnyAtomicType
+
 
 DEPENDENCY_TYPES = {'spec', 'feature', 'calendar', 'default-language',
                     'format-integer-sequence', 'language', 'limits',
                     'xml-version', 'xsd-version', 'unicode-version',
                     'unicode-normalization-form'}
 
-IGNORE_SPECS = {'XQ10', 'XQ10+', 'XP30', 'XP30+', 'XQ30',
-                'XQ30+', 'XP31', 'XP31+', 'XQ31', 'XQ31+'}
+IGNORE_SPECS = {'XQ10', 'XQ10+', 'XP30', 'XP30+', 'XQ30', 'XQ30+',
+                'XP31', 'XP31+', 'XQ31', 'XQ31+', 'XT30+'}
 
-SKIP_TESTS = [
+SKIP_TESTS = {
     'fn-subsequence__cbcl-subsequence-010',
     'fn-subsequence__cbcl-subsequence-011',
     'fn-subsequence__cbcl-subsequence-012',
@@ -71,6 +74,10 @@ SKIP_TESTS = [
 
     # For XP30+
     'fn-root__K-NodeRootFunc-2',  # includes a XPath 3.0 fn:generate-id()
+    'fn-codepoints-to-string__cbcl-codepoints-to-string-021',  # Too long ...
+    'fn-unparsed-text__fn-unparsed-text-038',  # Typo in filename
+    'fn-unparsed-text-lines__fn-unparsed-text-lines-038',  # Typo in filename
+    'fn-serialize__serialize-xml-015b',  # Do not raise, attribute is good
 
     # Unicode FULLY-NORMALIZATION not supported in Python's unicodedata
     'fn-normalize-unicode__cbcl-fn-normalize-unicode-001',
@@ -78,13 +85,17 @@ SKIP_TESTS = [
 
     # IMHO incorrect tests
     'fn-resolve-uri__fn-resolve-uri-9',  # URI scheme names are lowercase
-]
+}
 
+xpath_parser = XPath2Parser
 
 QT3_NAMESPACE = "http://www.w3.org/2010/09/qt-fots-catalog"
 
 namespaces = {'': QT3_NAMESPACE}
 
+INVALID_BASE_URL = 'http://www.w3.org/fots/unparsed-text/'
+effective_base_url = None
+
 
 @contextlib.contextmanager
 def working_directory(dirpath):
@@ -146,9 +157,18 @@ class Source(object):
                 self.xml = lxml.etree.parse(self.file)
             else:
                 self.xml = ElementTree.parse(self.file)
-                self.namespaces = {
-                    k: v for _, (k, v) in ElementTree.iterparse(self.file, events=('start-ns',))
-                }
+                self.namespaces = {}
+                dup_index = 1
+
+                for _, (prefix, uri) in ElementTree.iterparse(self.file, events=('start-ns',)):
+                    if prefix not in self.namespaces:
+                        self.namespaces[prefix] = uri
+                    elif prefix:
+                        self.namespaces[f'{prefix}{dup_index}'] = uri
+                        dup_index += 1
+                    else:
+                        self.namespaces[f'default{dup_index}'] = uri
+                        dup_index += 1
 
         except (ElementTree.ParseError, lxml.etree.XMLSyntaxError):
             self.xml = None
@@ -251,6 +271,7 @@ class TestSet(object):
         full_path = os.path.abspath(self.file)
         directory = os.path.dirname(full_path)
         filename = os.path.basename(full_path)
+
         with working_directory(directory):
             xml_root = ElementTree.parse(filename).getroot()
 
@@ -387,7 +408,7 @@ class TestCase(object):
             print()
         return self.result.validate(verbose)
 
-    def run_xpath_test(self, verbose=1, with_context=True):
+    def run_xpath_test(self, verbose=1, with_context=True, with_xpath_nodes=False):
         """
         Helper function to parse and evaluate tests with elementpath.
 
@@ -417,15 +438,25 @@ class TestCase(object):
                 schema = xmlschema.XMLSchema(environment.schema.filepath)
                 schema_proxy = schema.xpath_proxy
 
-        parser = XPath2Parser(
+        if not static_base_uri:
+            pass
+        elif static_base_uri.startswith(INVALID_BASE_URL):
+            static_base_uri = static_base_uri.replace(INVALID_BASE_URL, effective_base_url)
+
+        parser = xpath_parser(
             namespaces=test_namespaces,
             xsd_version=self.xsd_version,
             schema=schema_proxy,
             base_uri=static_base_uri,
         )
 
+        if self.test is not None:
+            xpath_expression = self.test.replace(INVALID_BASE_URL, effective_base_url)
+        else:
+            xpath_expression = None
+
         try:
-            root_node = parser.parse(self.test)  # static evaluation
+            root_node = parser.parse(xpath_expression)  # static evaluation
         except Exception as err:
             if isinstance(err, ElementPathError):
                 raise
@@ -453,7 +484,7 @@ class TestCase(object):
 
             for param in environment.params:
                 name = param['name']
-                value = XPath2Parser().parse(param['select']).evaluate()
+                value = xpath_parser().parse(param['select']).evaluate()
                 variables[name] = value
 
             for source in environment.sources.values():
@@ -478,7 +509,10 @@ class TestCase(object):
             context = XPathContext(root=root, **kwargs)
 
         try:
-            result = root_node.evaluate(context)
+            if with_xpath_nodes:
+                result = root_node.evaluate(context)
+            else:
+                result = root_node.get_results(context)
         except Exception as err:
             if isinstance(err, ElementPathError):
                 raise
@@ -517,7 +551,7 @@ class Result(object):
     :param test_case: the test-case that the result validator belongs to.
     """
     # Validation helper tokens
-    parser = XPath2Parser()
+    parser = xpath_parser()
     string_token = parser.parse('fn:string($result)')
     string_join_token = parser.parse('fn:string-join($result, " ")')
 
@@ -619,7 +653,7 @@ class Result(object):
         if isinstance(result, list) and len(result) == 1:
             result = result[0]
 
-        parser = XPath2Parser()
+        parser = xpath_parser(xsd_version=self.test_case.xsd_version)
         root_node = parser.parse(self.value)
         context = XPathContext(root=ElementTree.XML("<empty/>"))
         expected_result = root_node.evaluate(context)
@@ -642,47 +676,17 @@ class Result(object):
         if isinstance(result, list) and len(result) == 1:
             result = result[0]
 
-        if self.value == 'xs:anyURI':
-            type_check = isinstance(result, str)
-        elif self.value == 'xs:boolean':
-            type_check = isinstance(result, bool)
-        elif self.value == 'xs:date':
-            type_check = isinstance(result, elementpath.datatypes.Date10)
-        elif self.value == 'xs:double':
-            type_check = isinstance(result, float)
-        elif self.value == 'xs:dateTime':
-            type_check = isinstance(result, elementpath.datatypes.DateTime10)
-        elif self.value == 'xs:dayTimeDuration':
-            type_check = isinstance(result, elementpath.datatypes.DayTimeDuration)
-        elif self.value == 'xs:decimal':
-            type_check = isinstance(result, (int, decimal.Decimal)) and not isinstance(result, bool)
-        elif self.value == 'xs:float':
-            type_check = isinstance(result, float)
-        elif self.value == 'xs:integer':
-            type_check = isinstance(result, int) and not isinstance(result, bool)
-        elif self.value == 'xs:NCName':
-            type_check = isinstance(result, str)
-        elif self.value == 'xs:nonNegativeInteger':
-            type_check = isinstance(result, int) and not isinstance(result, bool)
-        elif self.value == 'xs:positiveInteger':
-            type_check = isinstance(result, int) and not isinstance(result, bool)
-        elif self.value == 'xs:string':
-            type_check = isinstance(result, str)
-        elif self.value == 'xs:time':
-            type_check = isinstance(result, elementpath.datatypes.Time)
-        elif self.value == 'xs:token':
-            type_check = isinstance(result, str)
-        elif self.value == 'xs:unsignedShort':
-            type_check = isinstance(result, int) and not isinstance(result, bool)
-        elif self.value == 'document-node()*':
-            type_check = isinstance(result, list) and all(hasattr(x, 'getroot') for x in result)
-        elif self.value.startswith('element('):
+        if self.value == 'function(*)':
+            type_check = isinstance(result, XPathFunction)
+        elif self.value.startswith('element(') and self.value[8] != ')':
             tag = self.value[8:self.value.index(')')]
             type_check = hasattr(result, 'tag') and result.tag == tag
+        elif not self.parser.is_sequence_type(self.value):
+            msg = " test-case {}: {!r} is not a valid sequence type"
+            print(msg.format(self.test_case.name, self.value))
+            type_check = False
         else:
-            msg = "unknown type in assert_type: %s (result type is %s), test-case %s"
-            print(msg % (self.value, str(type(result)), self.test_case.name))
-            sys.exit(1)
+            type_check = self.parser.match_sequence_type(result, self.value)
 
         if not type_check:
             self.report_failure(
@@ -780,14 +784,17 @@ class Result(object):
             self.report_failure(verbose, error=err)
             return False
 
-        try:
-            length = 1 if isinstance(result, (str, bytes)) else len(result)
-        except TypeError as err:
-            self.report_failure(verbose, error=err)
-            return False
+        if isinstance(result, AnyAtomicType):
+            length = 1
         else:
-            if int(self.value) == length:
-                return True
+            try:
+                length = len(result)
+            except TypeError as err:
+                self.report_failure(verbose, error=err)
+                return False
+
+        if int(self.value) == length:
+            return True
 
         self.report_failure(
             verbose, expected=int(self.value), value=length, xpath_result=result
@@ -801,13 +808,13 @@ class Result(object):
         the original test.
         """
         try:
-            result = self.test_case.run_xpath_test(verbose)
+            result = self.test_case.run_xpath_test(verbose, with_xpath_nodes=True)
         except (ElementPathError, ParseError, EvaluateError) as err:
             self.report_failure(verbose, error=err)
             return False
 
         variables = {'result': result}
-        parser = XPath2Parser()
+        parser = xpath_parser(xsd_version=self.test_case.xsd_version)
         root_node = parser.parse(self.value)
         context = XPathContext(root=ElementTree.XML("<empty/>"), variables=variables)
         if root_node.boolean_value(root_node.evaluate(context)) is True:
@@ -823,10 +830,13 @@ class Result(object):
             self.report_failure(verbose, error=err)
             return False
 
+        if isinstance(result, (str, bytes)):
+            result = result.strip()
+
         expression = "fn:deep-equal($result, (%s))" % self.value
         variables = {'result': result}
 
-        parser = XPath2Parser()
+        parser = xpath_parser(xsd_version=self.test_case.xsd_version)
         root_node = parser.parse(expression)
         context = XPathContext(root=ElementTree.XML("<empty/>"), variables=variables)
         if root_node.evaluate(context) is True:
@@ -886,6 +896,15 @@ class Result(object):
 
                 for prefix, uri in environment.namespaces.items():
                     ElementTree.register_namespace(prefix, uri)
+            else:
+                for prefix, uri in self.parser.namespaces.items():
+                    ElementTree.register_namespace(prefix, uri)
+
+        if self.value is not None:
+            expected = self.value
+        else:
+            with open(self.attrib['file']) as fp:
+                expected = fp.read()
 
         if type(result) == list:
             parts = []
@@ -896,11 +915,22 @@ class Result(object):
                     parts.append(str(item.value))
                 elif hasattr(item, 'tag'):
                     parts.append(tostring(item).decode('utf-8').strip())
+                elif hasattr(item, 'getroot'):
+                    parts.append(tostring(item.getroot()).decode('utf-8').strip())
                 else:
                     parts.append(str(item))
             xml_str = ''.join(parts)
         else:
-            xml_str = tostring(result.getroot()).decode('utf-8').strip()
+            try:
+                root = result.getroot()
+            except AttributeError:
+                root = result
+
+            xml_str = tostring(root).decode('utf-8').strip()
+
+        # Remove character data from result if expected result is serialized
+        if '\n' not in expected:
+            xml_str = '>'.join(s.lstrip() for s in xml_str.split('>\n'))
 
         # Strip the tail from serialized result
         if '>' in xml_str:
@@ -908,13 +938,7 @@ class Result(object):
             if tail_pos < len(xml_str):
                 xml_str = xml_str[:tail_pos]
 
-        if self.value is not None:
-            expected = self.value
-        else:
-            with open(self.attrib['file']) as fp:
-                expected = fp.read()
-
-        if xml_str == expected:
+        if xml_str == expected or xml_str.replace(' />', '/>') == expected:
             return True
 
         # 2nd tentative (expected result from a serialization)
@@ -956,6 +980,8 @@ def main():
                         help="run only test cases that have a specific XPath expression")
     parser.add_argument('-i', dest='ignore_case', action='store_true', default=False,
                         help="ignore character case for regex pattern matching")
+    parser.add_argument('--xp30', action='store_true', default=False,
+                        help="test XPath 3.0 parser")
     parser.add_argument('-l', '--lxml', dest='use_lxml', action='store_true', default=False,
                         help="use lxml.etree for environment sources (default is ElementTree)")
     parser.add_argument('-v', dest='verbose', action='count', default=1,
@@ -979,9 +1005,20 @@ def main():
         print("Error: catalog file %s does not exist" % args.catalog)
         sys.exit(1)
 
+    if args.xp30:
+        from elementpath.xpath30 import XPath30Parser
+
+        global xpath_parser
+        xpath_parser = XPath30Parser
+        IGNORE_SPECS.remove('XP30')
+        IGNORE_SPECS.remove('XP30+')
+
     with working_directory(dirpath=os.path.dirname(catalog_file)):
         catalog_xml = ElementTree.parse(catalog_file)
 
+        global effective_base_url
+        effective_base_url = 'file://{}/fn/unparsed-text/'.format(os.getcwd())
+
         environments = {}
         for child in catalog_xml.getroot().iterfind("environment", namespaces):
             environment = Environment(child, args.use_lxml)
@@ -1002,7 +1039,7 @@ def main():
 
         for test_set in test_sets.values():
             # ignore test cases for XQuery, and 3.0
-            ignore_all_in_test_set = any(
+            ignore_all_in_test_set = test_set.specs and all(
                 dep in IGNORE_SPECS for dep in test_set.specs
             )
 
@@ -1013,8 +1050,13 @@ def main():
                     count_skip += 1
                     continue
 
+                # ignore test cases for XML version 1.1 (not yet supported by Python's libraries)
+                if test_case.xml_version == '1.1':
+                    count_skip += 1
+                    continue
+
                 # ignore test cases for XQuery, and 3.0
-                if any(dep in IGNORE_SPECS for dep in test_case.specs):
+                if test_case.specs and all(dep in IGNORE_SPECS for dep in test_case.specs):
                     count_skip += 1
                     continue
 
@@ -1048,6 +1090,14 @@ def main():
                     count_skip += 1
                     continue
 
+                if args.xp30 and test_case.test:
+                    if 'parse-json' in test_case.test:
+                        count_skip += 1
+                        continue
+                    elif 'map {' in test_case.test:
+                        count_skip += 1
+                        continue
+
                 count_run += 1
                 try:
                     case_result = test_case.run(verbose=args.verbose)
